"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const pluralize_1 = require("pluralize");
const macroable_1 = require("macroable");
const lodash_1 = require("lodash");
const Route_1 = require("./Route");
class RouteResource extends macroable_1.Macroable {
    constructor(_resource, _controller, _globalMatchers, _shallow = false) {
        super();
        this._resource = _resource;
        this._controller = _controller;
        this._globalMatchers = _globalMatchers;
        this._shallow = _shallow;
        this.routes = [];
        this._buildRoutes();
    }
    _makeRoute(pattern, methods, action, baseName) {
        baseName = baseName.split('.').map((token) => lodash_1.snakeCase(token)).join('.');
        const route = new Route_1.Route(pattern, methods, `${this._controller}.${action}`, this._globalMatchers);
        route.as(`${baseName}.${action}`);
        this.routes.push(route);
    }
    _buildRoutes() {
        this._resource = this._resource.replace(/^\//, '').replace(/\/$/, '');
        const resourceTokens = this._resource.split('.');
        const baseName = resourceTokens.map((token) => lodash_1.snakeCase(token)).join('.');
        const mainResource = resourceTokens.pop();
        const fullUrl = `${resourceTokens
            .map((token) => {
            return `${token}/:${lodash_1.snakeCase(pluralize_1.singular(token))}_id`;
        })
            .join('/')}/${mainResource}`;
        const shallowUrl = this._shallow ? mainResource : fullUrl;
        this._makeRoute(fullUrl, ['GET'], 'index', baseName);
        this._makeRoute(`${fullUrl}/create`, ['GET'], 'create', baseName);
        this._makeRoute(fullUrl, ['POST'], 'store', baseName);
        this._makeRoute(`${shallowUrl}/:id`, ['GET'], 'show', baseName);
        this._makeRoute(`${shallowUrl}/:id/edit`, ['GET'], 'edit', baseName);
        this._makeRoute(`${shallowUrl}/:id`, ['PUT', 'PATCH'], 'update', baseName);
        this._makeRoute(`${shallowUrl}/:id`, ['DELETE'], 'destroy', baseName);
    }
    _filter(names, inverse) {
        return this.routes.filter((route) => {
            const match = names.find((name) => route.name.endsWith(name));
            return inverse ? !match : match;
        });
    }
    only(names) {
        this
            ._filter(names, true)
            .forEach((route) => (route.deleted = true));
        return this;
    }
    except(names) {
        this
            ._filter(names, false)
            .forEach((route) => (route.deleted = true));
        return this;
    }
    apiOnly() {
        return this.except(['.create', '.edit']);
    }
    middleware(middleware) {
        for (let name in middleware) {
            if (name === '*') {
                this.routes.forEach((route) => route.middleware(middleware[name]));
            }
            else {
                const route = this.routes.find((route) => route.name.endsWith(name));
                if (route) {
                    route.middleware(middleware[name]);
                }
            }
        }
        return this;
    }
    where(key, matcher) {
        this.routes.forEach((route) => {
            route.where(key, matcher);
        });
        return this;
    }
    namespace(namespace) {
        this.routes.forEach((route) => {
            route.namespace(namespace);
        });
        return this;
    }
}
exports.RouteResource = RouteResource;
RouteResource._macros = {};
RouteResource._getters = {};
